/**
 * $Id: beam.java.stub,v 1.2 1998/06/07 11:21:55 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates gleTwistExtrusion() in operation
 *
 * $Log: beam.java.stub,v $
 * Revision 1.2  1998/06/07 11:21:55  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.1  1998/04/30 23:33:38  descarte
 * Initial revision
 *
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class beamapp extends Applet implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The number of spine points in the beam */
    private int SPINEPOINTS = 22;

    /** Scale factor for the beam */
    private double TSCALE = 6.0;

    /** The vertex data array for the beam */
    private double[][] spinePoints = new double[SPINEPOINTS][3];

    /** The twist data array for the beam */
    private double[] spineTwists = new double[SPINEPOINTS];

    /** The number of points in the cross section */
    private int XSECTIONPOINTS = 12;

    /** Scale factor for the cross section */
    private double XSCALE = 0.1;

    /** The vertex data array for the cross section */
    private double[][] xsectionPoints = {
        { -6.0, 6.0 },
        { 6.0, 6.0 },
        { 6.0, 5.0 },
        { 1.0, 5.0 },
        { 1.0, -5.0 },
        { 6.0, -5.0 },
        { 6.0, -6.0 },
        { -6.0, -6.0 },
        { -6.0, -5.0 },
        { -1.0, -5.0 },
        { -1.0, 5.0 },
        { -6.0, 5.0 }
      };

    /** The normal data array for the cross section */
    private double[][] xsectionNormals = new double[XSECTIONPOINTS][2];

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        gle_ = coregle_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Beam" );
      }

    public String getVersion() {
        return new String( "$Id: beam.java.stub,v 1.2 1998/06/07 11:21:55 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the gleTwistExtrusion()\n" );
        sb.append( "method present in the GLE library. Drag the mouse about to twist\n" );
        sb.append( "the beam at each spine point along the extrusion.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        /**#windowListener */

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColor );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColor );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glColorMaterial( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );

        /** Initialize the spine data */
        for ( int i = 0 ; i < SPINEPOINTS ; i++ ) {
            spinePoints[i][0] = ( -1.1 + ( (double)i / 10.0 ) ) * TSCALE;
            spinePoints[i][1] = 0.0;
            spinePoints[i][2] = 0.0;

            spineTwists[i] = 0.0;
          }

        /** Initialize the cross section data */
        for ( int i = 0 ; i < XSECTIONPOINTS ; i++ ) {
            xsectionPoints[i][0] *= XSCALE;
            xsectionPoints[i][1] *= XSCALE;

            if ( i != 0 ) {
                double ax =
                    xsectionPoints[i][0] - xsectionPoints[i-1][0];
                double ay =
                    xsectionPoints[i][1] - xsectionPoints[i-1][1];
                double alen = 1.0 / Math.sqrt( ax * ax + ay * ay );
                ax *= alen;
                ay *= alen;
                xsectionNormals[i-1][0] = -ay;
                xsectionNormals[i-1][1] = ax;
              }
          }
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            gl_.glRotated( 43.0, 1.0, 0.0, 0.0 );
            gl_.glRotated( 43.0, 0.0, 1.0, 0.0 );
            gl_.glScaled( 1.8, 1.8, 1.8 );

            /** Draw the polybeam */
            gle_.gleTwistExtrusion( XSECTIONPOINTS, xsectionPoints,
                                    xsectionNormals, null,
                                    SPINEPOINTS, spinePoints, 
                                    null, spineTwists );
        gl_.glPopMatrix();
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().size().width;
        int height = evt.getComponent().size().height;
        int x = evt.getX();
        int y = evt.getY();

        for ( int i = 0 ; i < SPINEPOINTS ; i++ ) {
            double z = ((double)( i - 14 ) ) / 10.0;
            spineTwists[i] += ( x - beginx ) * Math.exp( -3.0 * z * z );
          }
        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
