/**
 * $Id: candlestick.java.stub,v 1.2 1998/06/07 11:21:55 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates gleLathe() in operation
 *
 * $Log: candlestick.java.stub,v $
 * Revision 1.2  1998/06/07 11:21:55  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.1  1998/05/01 14:48:11  descarte
 * Initial revision
 *
 * Revision 1.1  1998/05/01 11:50:39  descarte
 * Initial revision
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.GLTimesRoman24BitmapFont;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class candlestick extends Frame implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The number of spine points in the candlestick */
    private int CONTOURPOINTS = 100;

    /** Scale factor for the candlestick */
    private double TSCALE = 1.0;

    /** The vertex data array for the candlestick */
    private double[][] contourPoints = new double[CONTOURPOINTS][3];

    /** The normals for the contour */
    private double[][] contourNormals = new double[CONTOURPOINTS][2];

    /** Defines the lathe/spiral parameters */
    private double startRadius = 1.5;
    private double drdTheta = 0.0;
    private double startZ = 0.0;
    private double dzdTheta = 0.0;
    private double startTheta = 0.0;
    private double sweepTheta = 360.0;
    private double[][] startXForm = new double[2][3];
    private double[][] dXdTheta = new double[2][3];

    /** The up vector for the contour */
    private double[] up = { 1.0, 0.0, 0.0 };

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Current normal and join mode message */
    private String currentMode = new String( "Facet Edge Normal Vectors" );
    private int mode = -1;

    /** Font to draw the mode in */
    private GLTimesRoman24BitmapFont timesRoman24 =
        new GLTimesRoman24BitmapFont();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        candlestick d = new candlestick();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public candlestick() {
        gl_ = coregl_;
        glu_ = coreglu_;
        gle_ = coregle_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Candlestick" );
      }

    public String getVersion() {
        return new String( "$Id: candlestick.java.stub,v 1.2 1998/06/07 11:21:55 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the gleLathe()\n" );
        sb.append( "method present in the GLE library. Drag the mouse about to twist\n" );
        sb.append( "the shape. You can also change the normal generation style\n" );
        sb.append( "by hitting the 'm' or 'M' keys.\n\n" );
        sb.append( "Keys:\n" );
        sb.append( "    m,M - Toggle normal styles\n" );
        sb.append( "    ESC - Quit demo" );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        this.addWindowListener( this );

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColor = { 0.54f, 0.54f, 0.54f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColor = { 0.54f, 0.54f, 0.54f, 1.0f }; 

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, lightOneColor );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColor );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, lightOneColor );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColor );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, lightTwoColor );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_NORMALIZE );

        /** Initialize the contour data */
        int index = 0;
        addPoint( index++, -8.0, 0.0 );
        addPoint( index++, -10.0, 0.0 ); addNormal( index );
        addPoint( index++, -10.0, 2.0 ); addNormal( index );
        addPoint( index++, -9.6, 2.0 ); addNormal( index );
        addPoint( index++, -8.0, 0.0 ); addNormal( index );
        addPoint( index++, -5.8, 0.0 ); addNormal( index );
        addPoint( index++, -5.2, 0.6 ); addNormal( index );
        addPoint( index++, -4.6, 0.0 ); addNormal( index );
        addPoint( index++, -1.5, 0.0 ); addNormal( index );

        double dtheta = Math.PI / 14.0;
        double theta = 0;
        for ( int i = 0 ; i < 14 ; i++ ) {
            addPoint( index++, ( -1.5 * Math.cos( theta ) ), 
                      ( 1.5 * Math.sin( theta ) ) );
            contourNormals[index - 1][0] = -Math.cos( theta );
            contourNormals[index - 1][1] = Math.sin( theta );
            theta += dtheta;
          }

        addPoint( index++, 1.5, 0.0 ); addNormal( index );
        addPoint( index++, 4.6, 0.0 ); addNormal( index );
        addPoint( index++, 5.2, 0.6 ); addNormal( index );
        addPoint( index++, 5.8, 0.0 ); addNormal( index );
        addPoint( index++, 7.0, 0.0 ); addNormal( index );
        addPoint( index++, 7.5, 0.2 ); addNormal( index );
        addPoint( index++, 8.0, 0.8 ); addNormal( index );
        addPoint( index++, 8.3, 0.9 ); addNormal( index );
        addPoint( index++, 8.15, 1.8 ); addNormal( index );
        addPoint( index++, 8.8, 2.8 ); addNormal( index );
        addPoint( index++, 9.2, 3.8 ); addNormal( index );
        addPoint( index++, 9.5, 3.8 ); addNormal( index );
        addPoint( index++, 9.56, 3.75 ); addNormal( index );
        addPoint( index++, 9.62, 3.75 ); addNormal( index );
        addPoint( index++, 9.7, 3.8 ); addNormal( index );
        addPoint( index++, 10.0, 3.8 ); addNormal( index );
        addPoint( index++, 10.0, 0.0 ); addNormal( index );
        addPoint( index++, 7.0, 0.0 ); addNormal( index );

        /** Initialize the transforms */
        startXForm[0][0] = 1.0;
        startXForm[0][1] = 0.0;
        startXForm[0][2] = 0.0;
        startXForm[1][0] = 0.0;
        startXForm[1][1] = 1.0;
        startXForm[1][2] = 0.0;

        dXdTheta[0][0] = 0.0;
        dXdTheta[0][1] = 0.0;
        dXdTheta[0][2] = 0.0;
        dXdTheta[1][0] = 0.0;
        dXdTheta[1][1] = 0.0;
        dXdTheta[1][2] = 0.0;

        /** Set the material of the candlestick */
        float[] ambient = { 0.25f, 0.25f, 0.25f };
        float[] diffuse = { 0.8f, 0.6f, 0.175f };
        float[] specular = { 0.45f, 0.45f, 0.45f };
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, ambient );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, diffuse );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, specular );
        gl_.glMaterialf( GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, 2.0f );

        /** Set the GLE joining style */
        int style = gle_.gleGetJoinStyle();
        style &= ~GLE.TUBE_NORM_MASK;
        style |= GLE.TUBE_NORM_PATH_EDGE;
        style |= GLE.TUBE_NORM_FACET;
        gle_.gleSetJoinStyle( style );
      }

    /** Adds points into the contour points array */
    private void addPoint( int index, double x, double y ) {
        contourPoints[index][0] = x * TSCALE;
        contourPoints[index][1] = y * TSCALE;
      }

    /** Adds a normal into the normals array */
    private void addNormal( int index ) {
        double dx, dy, w;
        dx = contourPoints[index - 1][0];
        dx -= contourPoints[index - 2][0];
        dy = contourPoints[index - 1][1];
        dy -= contourPoints[index - 2][1];
        w = 1.0 / Math.sqrt( dx * dx + dy * dy );
        dx *= w;
        dy *= w;
        contourNormals[index - 1][0] = -dy;
        contourNormals[index - 1][1] = dx;
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            gl_.glRotated( -75.0, 1.0, 0.0, 0.0 );
            gl_.glRotated( -105.0, 0.0, 0.0, 1.0 );

            /** Enable lighting */
            gl_.glEnable( GL.GL_LIGHTING );

            /** Do the shearing */
            dzdTheta = -0.015 * ( beginy - 150.0 );

            /** Draw the lathe object */
            gle_.gleLathe( CONTOURPOINTS,
                           contourPoints,
                           contourNormals,
                           up,
                           startRadius,
                           drdTheta,
                           startZ,
                           dzdTheta,
                           startXForm,
                           dXdTheta,
                           startTheta,
                           sweepTheta );

            /** Disable lighting */
            gl_.glDisable( GL.GL_LIGHTING );

            /** Disable depth buffer writes */
            gl_.glDisable( GL.GL_DEPTH_TEST );

            /** Draw the current normal and join mode */
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );

            double[] modelViewMatrix = new double[16];
            double[] projectionMatrix = new double[16];
            int[] viewport = new int[4];
            double[] win = new double[4];

            gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );
            gl_.glGetDoublev( GL.GL_MODELVIEW_MATRIX, modelViewMatrix );
            gl_.glGetDoublev( GL.GL_PROJECTION_MATRIX, projectionMatrix );
            glu_.gluUnProject( 10, 10, 1, modelViewMatrix, projectionMatrix,
                               viewport, win );

            gl_.glRasterPos3d( win[0], win[1], win[2] );
            timesRoman24.drawString( currentMode );

            /** Re-enable the depth buffer */
            gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glPopMatrix();
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();

        System.err.println( "Mouse clicked at " + beginx + ", " + beginy );
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int x = evt.getX();
        int y = evt.getY();

        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'm' || evt.getKeyChar() == 'M' ) {
            int style = gle_.gleGetJoinStyle();
            switch ( mode ) {
                case -1: {
                    currentMode = new String( "Facet Normal Vectors" );
                    mode++;
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_FACET;
                    break;
                  }
                case 0: {
                    currentMode = new String( "Edge Normal Vectors" );
                    mode++;
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_EDGE;
                    break;
                  }
                case 1: {
                    currentMode = new String( "Facet Sweep Normal Vectors" );
                    mode++;
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_PATH_EDGE;
                    style |= GLE.TUBE_NORM_FACET;
                    break;
                  }
                case 2: {
                    currentMode = new String( "Edge Sweep Normal Vectors" );
                    mode = -1;
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_PATH_EDGE;
                    style |= GLE.TUBE_NORM_EDGE;
                    break;
                  }
              }
            gle_.gleSetJoinStyle( style );
            glc.repaint();
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
