/**
 * $Id: demoDumper.java,v 1.1 1998/05/02 01:43:43 descarte Exp descarte $
 *
 * Copyright (c)1997-1998 Arcane Technologies Ltd <http://www.arcana.co.uk>
 *
 * Standalone program that dumps all the version and description information
 * from the listed demo programs. HTML files, one per demo are also dumped
 * in the given target directory.
 *
 * $Log: demoDumper.java,v $
 * Revision 1.1  1998/05/02 01:43:43  descarte
 * Initial revision
 *
 */

package com.hermetica.gle.demos;

import java.io.*;
import java.util.*;
import com.hermetica.util3d.demoTemplate;

public class demoDumper {

    /** Version information */
    public static final String VERSION =
        new String( "$Id: demoDumper.java,v 1.1 1998/05/02 01:43:43 descarte Exp descarte $" );

    /** Usage string */
    private static final String USAGE = 
        new String( "Usage: demoDumper <html|htmlApplet|latex> <target_directory>" );

    /** A hashtable of all the demos we have, keyed on classname */
    private static Hashtable stdClassHash = new Hashtable();

    /** The target directory for the HTML files */
    private static String targetDirectory = null;

    /** The output stream for the index.html file */
    private static PrintStream indexStream;

    /** List of images */
    private static String[] imageList = null;

    /** The suffix of the output file */
    private static String suffix = null;
    private static String HTML = new String( ".html" );
    private static String LATEX = new String( ".tex" );

    private static final int htmlMode = 0x00;
    private static final int htmlAppletMode = 0x01;
    private static final int latexMode = 0x02;
    private static int mode = htmlMode;

    /** Create a new boot frame */
    public static void main( String argv[] ) {

        if ( argv.length != 2 ) {
            System.err.println( USAGE );
            System.exit( 1 );
          }

        /** Create the String containing the name of the target directory */
        targetDirectory = new String( argv[1] );

        /** Create the hashtable of classes */
        createHashes();

        /** Calculates the suffix of the files */
        if ( argv[0].equals( "html" ) ) {
            suffix = HTML;
            mode = htmlMode;
          } else {
            if ( argv[0].equals( "htmlApplet" ) ) {
                suffix = HTML;
                mode = htmlAppletMode;
              } else {
                suffix = LATEX;
                mode = latexMode;
              }
          }

        /** Scans the targetDirectory + "/img" directory for images */
        File imgDirectory = new File( targetDirectory + File.separator + "img" );
        if ( imgDirectory.isDirectory() ) {
            imageList = imgDirectory.list();
          }

        /** Create the 'index.html' file output stream */
        File outFile = 
            new File( targetDirectory + File.separator + "index" + suffix );

        try {
            indexStream =
                new PrintStream( new FileOutputStream( outFile ) );

            switch ( mode ) {
                case htmlMode:
                case htmlAppletMode: {
                    /** Write out the HTML for this demo */
                    indexStream.println( "<HTML>\n<HEAD>\n<TITLE>GLE Demos</TITLE>\n</HEAD>\n" );
                    indexStream.println( "<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LINK=\"#3a15ff\" ALINK=\"#ff0000\" VLINK=\"#ff282d\">\n" );
                    indexStream.println( "<CENTER><FONT SIZE=\"+2\"><B>GLE Demos</B></FONT></CENTER>\n" );
                    indexStream.println( "<P>\n<TABLE BORDER=0 WIDTH=\"100%\"><TR><TD VALIGN=top><CENTER><IMG SRC=\"../arcanelogos.gif\" WIDTH=82 HEIGHT=128 ALT=\"[ Pictish Glyph ]\">" );
                    indexStream.println( "<BR><FONT SIZE=\"-1\"><B>[ <A HREF=\"http://www.arcana.co.uk\">Arcane Technologies Ltd.</A> ]</B></FONT>" );
                    indexStream.println( "</CENTER></TD><TD>\n" );
                    break;
                  }
                case latexMode: {
                    indexStream.println( "\\documentstyle[]{article}\n\\parindent 0mm\n" );
                    indexStream.println( "\\pdfoutput=1\n\\relax\n" );
                    indexStream.println( "\\pdfpagewidth=8.26in\n\\pdfpageheight=11.69in\n" );
                    indexStream.println( "\\begin{document}\n" );
                    break;
                  }
              }
          } catch ( IOException e ) {
            e.printStackTrace();
          }

        /** Display all the standard demos */
        if ( mode == htmlMode || mode == htmlAppletMode ) {
            indexStream.println( "<H3>Standard Demonstrations</H3>\n<UL>\n" );
          }

        for ( Enumeration classEnum = stdClassHash.keys() ;
              classEnum.hasMoreElements() ; ) {
            String key = (String)classEnum.nextElement();
            String className = (String)stdClassHash.get( key );

            System.err.println( "Demo Name: " + className );
            dumpDemo( key, className );
          }

        if ( mode == htmlMode || mode == htmlAppletMode ) {
            indexStream.println( "</UL>\n<P>\n" );
          }

        /** Flush and close the index.html file output stream */

        if ( mode == htmlMode || mode == htmlAppletMode ) {
            indexStream.println( "</TD></TR></TABLE>\n</BODY></HTML>\n" );
            indexStream.println( "<!--#include virtual=\"/footer.html\" -->\n" );
          }

        indexStream.flush();
        indexStream.close();
      }

    /** Dumps a demo's data */
    private static void dumpDemo( String key, String className ) {

        /** Instantiate a new class of the given name */
        try {
            if ( mode == htmlAppletMode ) {
                className = new String( className + "app" );
              }

            demoTemplate tmpObject = 
                (demoTemplate)( Class.forName( className ) ).newInstance();

            switch ( mode ) {
                case htmlMode: {
                    writeHTML( key, className, tmpObject.getVersion(),
                               tmpObject.getDescription(),
                               tmpObject.getOriginalAuthor() );
                    writeHTMLIndex( key, className, tmpObject.getVersion() );
                    break;
                  }
                case htmlAppletMode: {
                    writeHTMLApplet( key, className, tmpObject.getVersion(),
                                     tmpObject.getDescription(),
                                     tmpObject.getOriginalAuthor(),
                                     tmpObject.getWidth(), 
                                     tmpObject.getHeight() );
                    writeHTMLIndex( key, className, tmpObject.getVersion() );
                    break;
                  }
                case latexMode: {
                    writeLaTeX( key, className, tmpObject.getVersion(),
                                tmpObject.getDescription() );
                    writeLaTeXIndex( key, className, tmpObject.getVersion() );
                    break;
                  }
              }
          } catch ( ClassNotFoundException e ) {
            e.printStackTrace();
          } catch ( InstantiationException e2 ) {
            e2.printStackTrace();
          } catch ( IllegalAccessException e3 ) {
            e3.printStackTrace();
          }
      }

    /** 
     * Writes out an 'index.html' file containing all the demos and 
     * hyperlinks to them
     */
    private static void writeHTMLIndex( String title, String className,
                                        String version ) {

        indexStream.println( "<LI><B><A HREF=\"" + className + ".html\" TARGET=\"demoFrame_\">" + title + "</A></B>\n" );
      }

    /**
     * Writes out an 'index.tex' file containing \input lines for all the
     * demos
     */
    private static void writeLaTeXIndex( String title, String className,
                                         String version ) {
        indexStream.println( "\\input " + className + ".tex\n" );
      }

    /** Writes an HTML file out with the given information */
    private static void writeHTML( String title, String className,
                                   String version, String description,
                                   String originalAuthor ) {

        File outFile = new File( targetDirectory + File.separator + className + HTML );

/*        if ( !outFile.canWrite() ) {
            System.err.println( "\t*** Cannot write to " + targetDirectory + File.separator + className + ".html for writing!" );
            return;
          } */

        try {
            PrintStream oStream = 
                new PrintStream( new FileOutputStream( outFile ) );

            /** Write out the HTML for this demo */
            oStream.println( "<HTML>\n<HEAD>\n<TITLE>GLE Demos: " + title + "</TITLE>\n</HEAD>\n" );
            oStream.println( "<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LINK=\"#3a15ff\" ALINK=\"#ff0000\" VLINK=\"#ff282d\">\n" );
            oStream.println( "<CENTER><FONT SIZE=\"+2\"><B>" + title + "</B></FONT></CENTER>\n" );
            oStream.println( "<P>\n<TABLE BORDER=0 WIDTH=\"100%\"><TR><TD VALIGN=top><CENTER><IMG SRC=\"../arcanelogos.gif\" WIDTH=82 HEIGHT=128 ALT=\"[ Pictish Glyph ]\">" );
            oStream.println( "<BR><FONT SIZE=\"-1\"><B>[ <A HREF=\"http://www.arcana.co.uk\">Arcane Technologies Ltd.</A> ]</B></FONT>" );
            oStream.println( "</CENTER></TD><TD>\n" );

            /** The body of the demo */
            oStream.println( "<DL><DT><B>Version:</B>\n" );
            oStream.println( "<DD>" + version + "<P>" );
            oStream.println( "<DT><B>Original Author:</B>\n" );

            /** Munge the originalAuthor line up into HREFs */
            char[] tmp = originalAuthor.toCharArray();
            char[] newtmp = new char[1024];
            int newIndex = 0;
            for ( int i = 0 ; i < tmp.length ; i++ ) {
                if ( tmp[i] == '<' ) {
                    newtmp[newIndex++] = '&';
                    newtmp[newIndex++] = 'l';
                    newtmp[newIndex++] = 't';
                    newtmp[newIndex] = ';';
                  } else {
                    if ( tmp[i] == '>' ) {
                        newtmp[newIndex++] = '&';
                        newtmp[newIndex++] = 'g';
                        newtmp[newIndex++] = 't';
                        newtmp[newIndex] = ';';
                      } else {
                        newtmp[newIndex] = tmp[i];
                      }
                  }
                newIndex++;
              }
            originalAuthor = new String( newtmp );

            oStream.println( "<DD>" + originalAuthor + "\n<P>\n" );
            oStream.println( "<DT><B>Class Name:</B>\n" );
            oStream.println( "<DD><TT>" + className + "</TT><P>\n" );
            oStream.println( "<DT><B>Description:</B>\n" );
            oStream.println( "<DD><PRE>" + description + "</PRE>\n" );
            oStream.println( "</DL>\n" );

            /** The image section... This looks in targetDirectory/img for images */
            if ( imageList != null ) {
                for ( int i = 0 ; i < imageList.length ; i++ ) {
                    if ( imageList[i].startsWith( className ) ) {
                        System.err.println( "\tLocated image" );
                        oStream.println( "<CENTER><IMG SRC=\"" +
                                         "img" + File.separator +
                                         imageList[i] + "\" ALT=\"[ Image ]\"></CENTER>\n<P>\n" );
                      }
                  }
              }

            /** The footer of the page */
            oStream.println( "</TD></TR></TABLE>\n" );
            oStream.println( "<!--#include virtual=\"/footer.html\" -->\n</BODY></HTML>\n" );

            /** Close off the output stream to the file */
            oStream.flush();
            oStream.close();
          } catch ( IOException e ) {
            e.printStackTrace();
          }
      }

    /** Writes an HTML file out with the given information as APPLET tags */
    private static void writeHTMLApplet( String title, String className,
                                         String version, String description,
                                         String originalAuthor,
                                         int width, int height ) {

        File outFile = new File( targetDirectory + File.separator + className + HTML );

/*        if ( !outFile.canWrite() ) {
            System.err.println( "\t*** Cannot write to " + targetDirectory + File.separator + className + ".html for writing!" );
            return;
          } */

        try {
            PrintStream oStream = 
                new PrintStream( new FileOutputStream( outFile ) );

            /** Write out the HTML for this demo */
            oStream.println( "<HTML>\n<HEAD>\n<TITLE>GLE Demos: " + title + "</TITLE>\n</HEAD>\n" );
            oStream.println( "<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LINK=\"#3a15ff\" ALINK=\"#ff0000\" VLINK=\"#ff282d\">\n" );
            oStream.println( "<CENTER><FONT SIZE=\"+2\"><B>" + title + "</B></FONT></CENTER>\n" );
            oStream.println( "<P>\n<TABLE BORDER=0 WIDTH=\"100%\"><TR><TD VALIGN=top><CENTER><IMG SRC=\"../arcanelogos.gif\" WIDTH=82 HEIGHT=128 ALT=\"[ Pictish Glyph ]\">" );
            oStream.println( "<BR><FONT SIZE=\"-1\"><B>[ <A HREF=\"http://www.arcana.co.uk\">Arcane Technologies Ltd.</A> ]</B></FONT>" );
            oStream.println( "</CENTER></TD><TD>\n" );

            /** The body of the demo */
            oStream.println( "<DL><DT><B>Version:</B>\n" );
            oStream.println( "<DD>" + version + "<P>" );
            oStream.println( "<DT><B>Class Name:</B>\n" );
            oStream.println( "<DD><TT>" + className + "</TT><P>\n" );

            /** Munge the originalAuthor line up into HREFs */
            char[] tmp = originalAuthor.toCharArray();
            char[] newtmp = new char[1024];
            int newIndex = 0;
            for ( int i = 0 ; i < tmp.length ; i++ ) {
                if ( tmp[i] == '<' ) {
                    newtmp[newIndex++] = '&';
                    newtmp[newIndex++] = 'l';
                    newtmp[newIndex++] = 't';
                    newtmp[newIndex] = ';';
                  } else {
                    if ( tmp[i] == '>' ) {
                        newtmp[newIndex++] = '&';
                        newtmp[newIndex++] = 'g';
                        newtmp[newIndex++] = 't';
                        newtmp[newIndex] = ';';
                      } else {
                        newtmp[newIndex] = tmp[i];
                      }
                  }
                newIndex++;
              }
            originalAuthor = new String( newtmp );

            oStream.println( "<DT><B>Original Author:</B>\n" );
            oStream.println( "<DD>" + originalAuthor + "\n<P>\n" );
            oStream.println( "<DT><B>Description:</B>\n" );
            oStream.println( "<DD><PRE>" + description + "</PRE>\n" );
            oStream.println( "</DL>\n" );

            /** Write out the APPLET tag to load up the demo in this page */
            oStream.println( "<CENTER><APPLET CODE=\"" +
                             className + "\" WIDTH=" +
                             (int)width + " HEIGHT=" + (int)height + "></APPLET>" );

            /** The footer of the page */
            oStream.println( "</TD></TR></TABLE>\n" );
            oStream.println( "<!--#include virtual=\"/footer.html\" -->\n</BODY></HTML>\n" );

            /** Close off the output stream to the file */
            oStream.flush();
            oStream.close();
          } catch ( IOException e ) {
            e.printStackTrace();
          }
      }

    /** Writes a LaTeX file out with the given information */
    private static void writeLaTeX( String title, String className,
                                    String version, String description ) {

        File outFile = new File( targetDirectory + File.separator + className + LATEX );

/*        if ( !outFile.canWrite() ) {
            System.err.println( "\t*** Cannot write to " + targetDirectory + File.separator + className + ".html for writing!" );
            return;
          } */

        try {
            PrintStream oStream = 
                new PrintStream( new FileOutputStream( outFile ) );

            /** Write out the HTML for this demo */
            oStream.println( "\\begin{center}\n{\\large GLE Demos: " + title + "}\n\\end{center}\n" );
//            oStream.println( "\\begin{tabular}{ll}\n" );
//            oStream.println( "Pictish Glyph & " );
//            oStream.println( "<P>\n<TABLE BORDER=0 WIDTH=\"100%\"><TR><TD VALIGN=top><CENTER><IMG SRC=\"/img/arcanelogos.gif\" WIDTH=82 HEIGHT=128 ALT=\"[ Pictish Glyph ]\">" );
//            oStream.println( "<BR><FONT SIZE=\"-1\"><B>[ <A HREF=\"/index.html\">Home</A> ]</B></FONT>" );
//            oStream.println( "<BR><FONT SIZE=\"-1\"><B>[ <A HREF=\"/products/magician/index.html\" TARGET=\"_top\">GLE</A> ]</B></FONT><BR>" );
//            oStream.println( "</CENTER></TD><TD>\n" );

            /** The body of the demo */
            oStream.println( "\\begin{description}\n\\item [{\\bf Version:}]\n" + 
                             "\\begin{verbatim}\n" + version + "\n\\end{verbatim}" );
            oStream.println( "\\item [{\\bf Class Name:}]\n" +
                             "\\begin{verbatim}\n" + className + "\n\\end{verbatim}\n" );
            oStream.println( "\\item [{\\bf Description:}]\n" + 
                             "\\vskip 4mm\n" + description );
            oStream.println( "\\end{description}\n" );

            /** The image section... This looks in targetDirectory/img for images */
            if ( imageList != null ) {
                for ( int i = 0 ; i < imageList.length ; i++ ) {
                    if ( imageList[i].startsWith( className ) &&
                         imageList[i].endsWith( ".png" ) ) {
                        System.err.println( "\tLocated image" );
                        oStream.println( "\\begin{center}\n" +
                                         "\\pdfimage\n" +
                                         "img" + File.separator + imageList[i] + "\n" );
                        oStream.println( "\\end{center}\n" +
                                         "\\vskip 5mm\n" );
                      }
                  }
              }

            /** The footer of the page */
//            oStream.println( "\\end{tabular}\n" );
//            oStream.println( "\\end{document}\n" );

            /** Close off the output stream to the file */
            oStream.flush();
            oStream.close();
          } catch ( IOException e ) {
            e.printStackTrace();
          }
      }

    /** Creates a hashtable of classnames and meaningful names */
    private static void createHashes() {
        /** Standard top-level demos */
        stdClassHash.put( new String( "Beam" ),
                          new String( "com.hermetica.gle.demos.beam" ) );
        stdClassHash.put( new String( "Candlestick" ),
                          new String( "com.hermetica.gle.demos.candlestick" ) );
        stdClassHash.put( new String( "Polycone" ),
                          new String( "com.hermetica.gle.demos.cone" ) );
        stdClassHash.put( new String( "Polycylinder" ),
                          new String( "com.hermetica.gle.demos.cylinder" ) );
        stdClassHash.put( new String( "Helix" ),
                          new String( "com.hermetica.gle.demos.helix1" ) );
        stdClassHash.put( new String( "Springy Helix" ),
                          new String( "com.hermetica.gle.demos.helix2" ) );
        stdClassHash.put( new String( "Stretchy Helix" ),
                          new String( "com.hermetica.gle.demos.helix3" ) );
        stdClassHash.put( new String( "Squashy Helix" ),
                          new String( "com.hermetica.gle.demos.helix4" ) );
        stdClassHash.put( new String( "Saxophone" ),
                          new String( "com.hermetica.gle.demos.horn" ) );
        stdClassHash.put( new String( "Cone Join Styles" ),
                          new String( "com.hermetica.gle.demos.joincone" ) );
        stdClassHash.put( new String( "Join Offsets" ),
                          new String( "com.hermetica.gle.demos.joinoffset" ) );
        stdClassHash.put( new String( "Cylinder Join Styles" ),
                          new String( "com.hermetica.gle.demos.joinstyle" ) );
        stdClassHash.put( new String( "Screw" ),
                          new String( "com.hermetica.gle.demos.screw" ) );
        stdClassHash.put( new String( "Taper" ),
                          new String( "com.hermetica.gle.demos.taper" ) );
        stdClassHash.put( new String( "Texas Branding Iron" ),
                          new String( "com.hermetica.gle.demos.texas" ) );
        stdClassHash.put( new String( "Spiral Transports" ),
                          new String( "com.hermetica.gle.demos.transport" ) );
        stdClassHash.put( new String( "Twistoid Corrugated Surface" ),
                          new String( "com.hermetica.gle.demos.twoid" ) );
      }
  }
