/**
 * $Id: helix.java.stub,v 1.2 1998/06/07 11:21:56 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates gleHelicoid() in operation
 *
 * $Log: helix.java.stub,v $
 * Revision 1.2  1998/06/07 11:21:56  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.1  1998/05/01 15:24:05  descarte
 * Initial revision
 *
 *
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class helixapp extends Applet implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** Current quaternion angle of rotation */
    private float[] curQuat = new float[4];

    /** Previous quaternion angle of rotation */
    private float[] lastQuat = new float[4];

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Trackball object used for calculating spin angles from mouse motion */
    private trackball ball = new trackball();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        gle_ = coregle_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Helix" );
      }

    public String getVersion() {
        return new String( "$Id: helix.java.stub,v 1.2 1998/06/07 11:21:56 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the gleHelicoid()\n" );
        sb.append( "method present in the GLE library. You can drag the mouse\n" );
        sb.append( "to spin the object about.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        /**#windowListener */

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColor );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColor );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glColorMaterial( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );

        /** Set the GLE join style */
        gle_.gleSetJoinStyle( GLE.TUBE_NORM_EDGE | GLE.TUBE_JN_ANGLE |
                              GLE.TUBE_JN_CAP );

        /** Set the colour of the shape */
        gl_.glColor3f( 0.6f, 0.8f, 0.3f );

        /** Create a new trackball for twiddling the shape */
        curQuat = ball.buildQuaternion( 0.0f, 0.0f, 0.0f, 0.0f );
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Matrix to rotate the object by as generated by a quaternion */
        float[][] m = new float[4][4];

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            m = ball.buildMatrix( curQuat );
            gl_.glMultMatrixf( m );

            /** Draw the helix */
            gle_.gleHelicoid( 1.0, 6.0, 2.0, -3.0, 4.0, 
                              null, null, 0.0, 1080.0 );
        gl_.glPopMatrix();
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().size().width;
        int height = evt.getComponent().size().height;
        int x = evt.getX();
        int y = evt.getY();

        lastQuat =
            ball.buildQuaternion( (float)( 2.0f * beginx - width ) / (float)width,
                                  (float)( height - 2.0f * beginy ) / (float)height,
                                  (float)( 2.0f * x - width ) / (float)width,
                                  (float)( height - 2.0f * y ) / (float)height );

        curQuat = ball.addQuats( lastQuat, curQuat ); 
        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
