/**
 * $Id: taper.java.stub,v 1.2 1998/06/07 11:22:04 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates gleSuperExtrusion() in operation
 *
 * $Log: taper.java.stub,v $
 * Revision 1.2  1998/06/07 11:22:04  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.1  1998/05/01 11:50:39  descarte
 * Initial revision
 *
 * Revision 1.1  1998/04/30 23:33:38  descarte
 * Initial revision
 *
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class taper extends Frame implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The number of spine points in the taper */
    private int CONTOURPOINTS = 25;

    /** Scale factor for the taper */
    private double TSCALE = 3.3333;

    /** The vertex data array for the taper */
    private double[][] contourPoints = {
        { 1.0, 1.0 },
        { 1.0, 2.9 },
        { 0.9, 3.0 },
        { -0.9, 3.0 },
        { 1.0, 2.9 },

        { -1.0, 1.0 },
        { -2.9, 1.0 },
        { -3.0, 0.9 },
        { -3.0, -0.9 },
        { -2.9, -1.0 },

        { -1.0, -1.0 },
        { -1.0, -2.9 },
        { -0.9, -3.0 },
        { 0.9, -3.0 },
        { 1.0, -2.9 },

        { 1.0, -1.0 },
        { 2.9, -1.0 },
        { 3.0, -0.9 },
        { 3.0, 0.9 },
        { 2.9, 1.0 },

        { 1.0, 1.0 }  /* repeat so that last normal is computed */
      };

    /** The normals for the contour */
    private double[][] contourNormals = new double[CONTOURPOINTS][2];

    /** The number of twist and taper points */
    private int PSIZE = 40;

    /** The path */
    private double[][] pathPoints = new double[PSIZE][3];

    /** The twist array */
    private double[] twist = new double[PSIZE];

    /** The taper array */
    private double[] taper = new double[PSIZE];

    /** Mouse tracking variables */
    private int beginx = 121,
                beginy = 121;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        taper d = new taper();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public taper() {
        gl_ = coregl_;
        gle_ = coregle_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Taper" );
      }

    public String getVersion() {
        return new String( "$Id: taper.java.stub,v 1.2 1998/06/07 11:22:04 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the gleSuperExtrusion()\n" );
        sb.append( "method present in the GLE library. Drag the mouse up and down to twist\n" );
        sb.append( "the shape and left and right to taper it.\n\n" );
        sb.append( "To quit this demo, hit the ``Escape'' key in the demo window." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        this.addWindowListener( this );

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColor = { 0.99f, 0.99f, 0.99f, 1.0f }; 

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColor );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColor );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glColorMaterial( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE );
        gl_.glEnable( GL.GL_COLOR_MATERIAL );

        /** Initialize the contour data */
        for ( int i = 0 ; i < contourPoints.length ; i++ ) {
            contourPoints[i][0] *= TSCALE;
            contourPoints[i][1] *= TSCALE;

            if ( i != 0 ) {
                double ax =
                    contourPoints[i][0] - contourPoints[i-1][0];
                double ay =
                    contourPoints[i][1] - contourPoints[i-1][1];
                double alen = 1.0 / Math.sqrt( ax * ax + ay * ay );
                ax *= alen;
                ay *= alen;
                contourNormals[i-1][0] = ay;
                contourNormals[i-1][1] = -ax;
              }
          }

        /** Initialize the twist and taper data */
        double z = -10.0,
               dz = 0.5;
        double ang = 0.0,
               dang = 20.0;
        for ( int i = 0 ; i < PSIZE ; i++ ) {
            pathPoints[i][0] = 0x0;
            pathPoints[i][1] = 0x0;
            pathPoints[i][2] = z;

            twist[i] = ang;
            ang += dang;

            taper[i] = 0.1 * Math.sqrt( 9.51 * 9.51 - z * z );
            z += dz;
          }
        taper[0] = taper[1];
        taper[39] = taper[38];

        /** Set the GLE joining style */
        gle_.gleSetJoinStyle( GLE.TUBE_JN_CAP | GLE.TUBE_CONTOUR_CLOSED |
                              GLE.TUBE_NORM_FACET | GLE.TUBE_JN_ANGLE );

        /** Set the colour */
        gl_.glColor3f( 0.5f, 0.6f, 0.6f );
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            gl_.glRotated( 130.0, 0.0, 1.0, 0.0 );
            gl_.glRotated( 65.0, 1.0, 0.0, 0.0 );

            /** Calculate the twist angles */
            double ang = 0.0,
                   dang = (double)beginy / 40.0;
            double z = -1.0,
                   dz = 1.999 / 38;
            double ponent = Math.abs( (double)beginx / 540.0 );
            for ( int i = 1 ; i < PSIZE - 1 ; i++ ) {
                twist[i] = ang;
                ang += dang;

                taper[i] = Math.pow( ( 1.0 - Math.pow( Math.abs( z ), 1.0 / ponent ) ), ponent );
                z += dz;
              }

            /** Calculate the transformation array */
            double[][][] transformArray = new double[PSIZE][2][3];
            for ( int i = 0 ; i < PSIZE ; i++ ) {
                double angle =
                    ( Math.PI / 180.0 ) * twist[i];
                double si = Math.sin( angle );
                double co = Math.cos( angle );
                transformArray[i][0][0] = taper[i] * co;
                transformArray[i][0][1] = -taper[i] * si;
                transformArray[i][0][2] = 0.0;
                transformArray[i][1][0] = taper[i] * si;
                transformArray[i][1][1] = taper[i] * co;
                transformArray[i][1][2] = 0.0;
              }

            /** Draw the polytaper */
            gle_.gleSuperExtrusion( 20, contourPoints,
                                    contourNormals, null,
                                    PSIZE, pathPoints, 
                                    null, transformArray );
        gl_.glPopMatrix();
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int width = evt.getComponent().size().width;
        int height = evt.getComponent().size().height;
        int x = evt.getX();
        int y = evt.getY();

        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
