/**
 * $Id: transport.java.stub,v 1.1 1998/06/07 11:22:04 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates gleLathe() in operation
 *
 * $Log: transport.java.stub,v $
 * Revision 1.1  1998/06/07 11:22:04  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.1  1998/05/01 14:48:11  descarte
 * Initial revision
 *
 * Revision 1.1  1998/05/01 11:50:39  descarte
 * Initial revision
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.GLTimesRoman24BitmapFont;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class transportapp extends Applet implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** The number of spine points in the transport */
    private int CONTOURPOINTS = 100;

    /** Scale factor for the transport */
    private double TSCALE = 1.80;

    /** The vertex data array for the transport */
    private double[][] contourPoints = new double[CONTOURPOINTS][3];

    /** The normals for the contour */
    private double[][] contourNormals = new double[CONTOURPOINTS][2];

    /** Defines the lathe/spiral parameters */
    private double startRadius = 7.3;
    private double drdTheta = 0.0;
    private double startZ = 0.0;
    private double dzdTheta = 0.0;
    private double startTheta = 0.0;
    private double sweepTheta = 90.0;

    /** The up vector for the contour */
    private double[] up = { 0.0, 0.0, 1.0 };

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;



    public void init() {
        gl_ = coregl_;
        glu_ = coreglu_;
        gle_ = coregle_;
        startDemo();
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Spiral Transports" );
      }

    public String getVersion() {
        return new String( "$Id: transport.java.stub,v 1.1 1998/06/07 11:22:04 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the use of the gleLathe()\n" );
        sb.append( "and gleSpiral() methods present in the GLE library. The upper\n" );
        sb.append( "shape is drawn with gleLathe() whereas the lower is drawn with\n" );
        sb.append( "gleSpiral(). As you move the mouse about the lower shape will\n" );
        sb.append( "twist upwards whereas the upper will not.\n\n" );
        sb.append( "Hit ``Escape'' to quit the demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        /**#layout */

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        /**#windowListener */

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColour = { 0.54f, 0.54f, 0.54f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColour = { 0.54f, 0.54f, 0.54f, 1.0f }; 

        float[] lightThreePosition = { 40.0f, 40.0f, -100.0f, 0.0f };
        float[] lightThreeColour = { 0.54f, 0.54f, 0.54f, 1.0f };

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_AMBIENT, lightOneColour );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColour );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, lightOneColour );
        gl_.glEnable( GL.GL_LIGHT0 );

        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColour );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_AMBIENT, lightTwoColour );
        gl_.glEnable( GL.GL_LIGHT1 );

        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_POSITION, lightThreePosition );
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_DIFFUSE, lightThreeColour );
        gl_.glLightfv( GL.GL_LIGHT2, GL.GL_AMBIENT, lightThreeColour );
        gl_.glEnable( GL.GL_LIGHT2 );

        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_NORMALIZE );

        /** Initialize the contour data */
        int index = 0;
        addPoint( index++, -1.0, -2.0 );
        addPoint( index++, -1.0, 0.0 ); addNormal( index );
        addPoint( index++, -2.0, 0.0 ); addNormal( index );
        addPoint( index++, 0.0, 2.0 ); addNormal( index );
        addPoint( index++, 2.0, 0.0 ); addNormal( index );
        addPoint( index++, 1.0, 0.0 ); addNormal( index );
        addPoint( index++, 1.0, -2.0 ); addNormal( index );

        CONTOURPOINTS = index;

        /** Set the material of the geometries */
        float[] ambient = { 0.15f, 0.15f, 0.15f };
        float[] diffuse = { 0.15f, 0.55f, 0.55f };
        float[] specular = { 0.4f, 0.4f, 0.4f };
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, ambient );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, diffuse );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR, specular );
        gl_.glMaterialf( GL.GL_FRONT_AND_BACK, GL.GL_SHININESS, 2.0f );

        /** Set the GLE joining style */
        int style = 0;
        style |= GLE.TUBE_JN_CAP;
        style |= GLE.TUBE_NORM_PATH_EDGE;
        style |= GLE.TUBE_NORM_FACET;
        style |= GLE.TUBE_CONTOUR_CLOSED;
        gle_.gleSetJoinStyle( style );
      }

    /** Adds points into the contour points array */
    private void addPoint( int index, double x, double y ) {
        contourPoints[index][0] = x * TSCALE;
        contourPoints[index][1] = y * TSCALE;
      }

    /** Adds a normal into the normals array */
    private void addNormal( int index ) {
        double dx, dy, w;
        dx = contourPoints[index - 1][0];
        dx -= contourPoints[index - 2][0];
        dy = contourPoints[index - 1][1];
        dy -= contourPoints[index - 2][1];
        w = 1.0 / Math.sqrt( dx * dx + dy * dy );
        dx *= w;
        dy *= w;
        contourNormals[index - 1][0] = -dy;
        contourNormals[index - 1][1] = dx;
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Wongle the spiral values depending on the mouse position */
        sweepTheta = 180.0 + 0.13 * beginx;
        dzdTheta = 0.03 * ( beginy + 10.0 );

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            gl_.glRotated( -75.0, 1.0, 0.0, 0.0 );
            gl_.glRotated( -10.0, 0.0, 0.0, 1.0 );

            /** 
             * First, draw with the routine that uses a local coordinate
             * system with torsion
             */
            gl_.glTranslatef( -0.5f, -0.5f, 4.2f );

            /** Draw the lathe object */
            gle_.gleLathe( CONTOURPOINTS,
                           contourPoints,
                           contourNormals,
                           up,
                           startRadius,
                           drdTheta,
                           startZ,
                           dzdTheta,
                           null,
                           null,
                           startTheta,
                           sweepTheta );

            /** 
             * Next, draw with the routine that uses the parallel transport
             */
            gl_.glTranslatef( 0.0f, 0.0f, -8.4f );
            gle_.gleSpiral( CONTOURPOINTS,
                            contourPoints,
                            contourNormals,
                            up,
                            startRadius,
                            drdTheta,
                            startZ,
                            dzdTheta,
                            null,
                            null,
                            startTheta,
                            sweepTheta );
        gl_.glPopMatrix();
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();

        System.err.println( "Mouse clicked at " + beginx + ", " + beginy );
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int x = evt.getX();
        int y = evt.getY();

        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            /**#dispose */
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        /**#dispose */
      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
