/**
 * $Id: twoid.java.stub,v 1.1 1998/06/07 11:22:08 descarte Exp descarte $
 *
 * Copyright (c)1998 Alligator Descartes <descarte@arcana.co.uk>
 *
 * Basic test program that demonstrates glePolyCone() in operation
 *
 * $Log: twoid.java.stub,v $
 * Revision 1.1  1998/06/07 11:22:08  descarte
 * Converted from COM.hermetica.* hierarchy to com.hermetica.*
 *
 * Revision 1.2  1998/04/30 22:56:47  descarte
 * Minor fix-ups.
 *
 * Revision 1.1  1998/04/30 21:33:34  descarte
 * Initial revision
 *
 */

package com.hermetica.gle.demos;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.magician.fonts.GLTimesRoman24BitmapFont;
import com.hermetica.util3d.*;
import com.hermetica.gle.*;

public class twoid extends Frame implements demoTemplate, GLComponentListener, KeyListener, WindowListener, MouseListener, MouseMotionListener {

    /** The OpenGL state machine */
    GL gl_ = null;
    CoreGL coregl_ = new CoreGL();

    GLU glu_ = null;
    CoreGLU coreglu_ = new CoreGLU();

    GLE gle_ = null;
    CoreGLE coregle_ = new CoreGLE();

    /** The OpenGL-aware Component we are rendering into */
    GLComponent glc = null;

    /** The dimensions of the demo */
    private int WIDTH = 300,
                HEIGHT = 300;

    /** Scaling value for the spine points */
    private double SCALE = 6.0;

    /** The vertex data array for the polytwoid */
    private double[][] toidPoints = {
        { -1.1, 0.0, 0.0 },
        { -1.0, 0.0, 0.0 },
        { 0.0, 0.0, 0.0 },
        { 1.0, 0.0, 0.0 },
        { 1.1, 0.0, 0.0 },
      };

    /** The colour data array for the polytwoid */
    private float[][] toidColours = {
        { 0.8f, 0.8f, 0.5f },
        { 0.8f, 0.4f, 0.5f },
        { 0.8f, 0.8f, 0.3f },
        { 0.4f, 0.4f, 0.5f },
        { 0.8f, 0.8f, 0.5f }
      };

    /** The twist array */
    private double[] toidTwists = {
        0.0, 0.0, 0.0, 0.0, 0.0
      };

    /** The number of twists to be applied at each segment */
    private int TWISTPOINTS = 20;
    private double[][] twistPoints = new double[TWISTPOINTS][2];
    private double[][] twistNormals = new double[TWISTPOINTS][2];

    /** The mode that the cone is being drawn in */
    private int mode = 0;
    private int normalMode = -1;
    private String currentMode = new String( "Raw Join Style" );
    private GLTimesRoman24BitmapFont timesRoman24 = 
        new GLTimesRoman24BitmapFont();

    /** The up vector */
    private double[] up = { 1.0, 0.0, 0.0 };

    /** Mouse tracking variables */
    private int beginx = 0,
                beginy = 0;

    /** Trackball object used for calculating spin angles from mouse motion */
    private trackball ball = new trackball();

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        twoid d = new twoid();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public twoid() {
        gl_ = coregl_;
        glu_ = coreglu_;
        gle_ = coregle_;
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Twistoid Corrugated Surface" );
      }

    public String getVersion() {
        return new String( "$Id: twoid.java.stub,v 1.1 1998/06/07 11:22:08 descarte Exp descarte $" );
      }

    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration illustrates the drawing of corrugated\n" );
        sb.append( "twistable surfaces with GLE.\n\n" );
        sb.append( "Hit ``Escape'' to quit this demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Linas Vepstas <linas@linas.org>" );
      }

    public void startDemo() {
        setLayout( new BorderLayout() );

        /** Create the OpenGL Component */
        glc = GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Add the key listener to the component */
        glc.addKeyListener( this );

        /** Add the mouse listeners to the component */
        glc.addMouseListener( this );
        glc.addMouseMotionListener( this );

        /** Add the window listener */
        this.addWindowListener( this );

        /** Add the listener and initialize the component */
        glc.addGLComponentListener( this );
        glc.initialize();
      }

    public void initialize( GLDrawable component ) {

        /** Set the clear colour */
        gl_.glClearColor( 0.2f, 0.2f, 0.2f, 1.0f );

        /** Initialize some OpenGL stuff */
        gl_.glClearDepth( 1.0f );
        gl_.glEnable( GL.GL_DEPTH_TEST );
        gl_.glShadeModel( GL.GL_SMOOTH );

        /** Initialize some lighting */
        float[] lightOnePosition = { 40.0f, 40f, 100.0f, 0.0f };
        float[] lightOneColour = { 0.89f, 0.89f, 0.89f, 1.0f }; 

        float[] lightTwoPosition = { -40.0f, 40f, 100.0f, 0.0f };
        float[] lightTwoColour = { 0.89f, 0.89f, 0.89f, 1.0f }; 

        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_POSITION, lightOnePosition );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_DIFFUSE, lightOneColour );
        gl_.glLightfv( GL.GL_LIGHT0, GL.GL_SPECULAR, lightOneColour );
        gl_.glEnable( GL.GL_LIGHT0 );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_POSITION, lightTwoPosition );
        gl_.glLightfv( GL.GL_LIGHT1, GL.GL_DIFFUSE, lightTwoColour );
        gl_.glEnable( GL.GL_LIGHT1 );
        gl_.glEnable( GL.GL_LIGHTING );

        /** Material set up */
        float[] material = { 0.93f, 0.79f, 0.93f, 1.0f };

        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, material );
        gl_.glMaterialfv( GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, material );

        /** Set up the extrusion values */
        for ( int i = 0 ; i < toidPoints.length ; i++ ) {
            toidPoints[i][0] *= SCALE;
            toidPoints[i][1] *= SCALE;
            toidPoints[i][2] *= SCALE;
          }

        /** Set up the twist values for each segment */
        for ( int i = 0 ; i < 11 ; i++ ) {
            double angle = Math.PI * ( (double)i / 10.0 );
            double co = Math.cos( angle );
            double si = Math.sin( angle );
            double TSCALE = 0.6;

            twistPoints[i][0] = TSCALE * ( -7 - 3.0 * co );
            twistPoints[i][1] = TSCALE * 1.8 * si;

            if ( i != 0 ) {
                double ax = twistPoints[i][0] - twistPoints[i - 1][0];
                double ay = twistPoints[i][1] - twistPoints[i - 1][1];
                double alen = 1.0 / Math.sqrt( ax * ax + ay * ay );
                twistNormals[i - 1][0] = -ay;
                twistNormals[i - 1][1] = ax;
              }
          }

        int i = 11;
        while ( i < TWISTPOINTS - 1 ) {
            double TSCALE = 0.6;
            twistPoints[i][0] = TSCALE * ( -10.0 + (double)i );
            twistPoints[i][1] = TSCALE * 0.0;

            double ax = twistPoints[i][0] - twistPoints[i - 1][0];
            double ay = twistPoints[i][1] - twistPoints[i - 1][1];
            double alen = 1.0 / Math.sqrt( ax * ax + ay * ay );
            twistNormals[i - 1][0] = -ay;
            twistNormals[i - 1][1] = ax;
            i++;

            twistPoints[i][0] = TSCALE * ( -9.5 + (double)i );
            twistPoints[i][1] = TSCALE * 1.0;

            ax = twistPoints[i][0] - twistPoints[i - 1][0];
            ay = twistPoints[i][1] - twistPoints[i - 1][1];
            alen = 1.0 / Math.sqrt( ax * ax + ay * ay );
            twistNormals[i - 1][0] = -ay;
            twistNormals[i - 1][1] = ax;
            i++;
          }

        /** Set the GLE join style */
        int style = gle_.gleGetJoinStyle();
        style &= ~GLE.TUBE_CONTOUR_CLOSED;
        gle_.gleSetJoinStyle( style );
      }

    public void reshape( GLDrawable component, int width, int height ) {
        gl_.glViewport( component, 0, 0, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -9.0, 9.0, -9.0, 9.0, 50.0, 150.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
        gl_.glLoadIdentity();
      }
        
    public void display( GLDrawable component ) {

        /** Update the twist values */
        toidTwists[2] = (double)( beginx - 121 ) / 8.0;
        toidPoints[3][0] = 1.0;
        toidPoints[3][1] = -( (double)( beginy - 121 ) / 200.0 );
        toidPoints[4][0] = 1.1;
        toidPoints[4][1] = -1.1 * ( (double)( beginy - 121 ) / 200.0 );

        /** Clear some buffers */
        gl_.glClear( GL.GL_DEPTH_BUFFER_BIT | GL.GL_COLOR_BUFFER_BIT );

        /** Set up some matrices to move the object into the frustum */
        gl_.glPushMatrix();
            gl_.glTranslatef( 0.0f, 0.0f, -80.0f );
            gl_.glRotatef( 43.0f, 1.0f, 0.0f, 0.0f );
            gl_.glRotatef( 43.0f, 0.0f, 1.0f, 0.0f );
            gl_.glScaled( 1.8, 1.8, 1.8 );

            /** Draw the polytwoid */
            gle_.gleTwistExtrusion( TWISTPOINTS, twistPoints,
                                    twistNormals, null,
                                    toidPoints.length,
                                    toidPoints, null, toidTwists );
        gl_.glPopMatrix();

        /** Disable lighting and depth buffering */
        gl_.glDisable( GL.GL_DEPTH_TEST );
        gl_.glDisable( GL.GL_LIGHTING );

        /** Calculate the raster position of the bottom-left-ish corner */
        double[] modelViewMatrix = new double[16];
        double[] projectionMatrix = new double[16];
        int[] viewport = new int[4];
        double[] win = new double[4];

        gl_.glGetIntegerv( GL.GL_VIEWPORT, viewport );
        gl_.glGetDoublev( GL.GL_MODELVIEW_MATRIX, modelViewMatrix );
        gl_.glGetDoublev( GL.GL_PROJECTION_MATRIX, projectionMatrix );
        glu_.gluUnProject( 10, 10, 1, modelViewMatrix, projectionMatrix,
                           viewport, win );
        gl_.glRasterPos3d( win[0], win[1], win[2] );

        /** Draw the string of text there */
        gl_.glColor3f( 1.0f, 1.0f, 1.0f );
        timesRoman24.drawString( currentMode );

        /** Re-enable lighting and depth-buffering */
        gl_.glEnable( GL.GL_LIGHTING );
        gl_.glEnable( GL.GL_DEPTH_TEST );
      }

    public GL getGL() {
        return coregl_;
      }

    /** Handles mouse events */
    public void mouseClicked( MouseEvent evt ) {
      }

    public void mouseEntered( MouseEvent evt ) {
      }

    public void mouseExited( MouseEvent evt ) {
      }

    public void mousePressed( MouseEvent evt ) {
        beginx = evt.getX();
        beginy = evt.getY();
      }

    public void mouseReleased( MouseEvent evt ) {
      }

    /** MouseMotionListener */
    public void mouseDragged( MouseEvent evt ) {
        int x = evt.getX();
        int y = evt.getY();

        glc.repaint();

        /** Update the mouse coordinates */
        beginx = x;
        beginy = y;
      }

    public void mouseMoved( MouseEvent evt ) {
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
        if ( evt.getKeyChar() == 'm' || evt.getKeyChar() == 'M' ) {
            int style = gle_.gleGetJoinStyle();
            mode++;
            switch ( mode ) {
                case 0: {
                    style &= ~GLE.TUBE_JN_MASK;
                    style |= GLE.TUBE_JN_RAW;
                    currentMode = new String( "Raw Join Style" );
                    break;
                  }
                case 1: {
                    style &= ~GLE.TUBE_JN_MASK;
                    style |= GLE.TUBE_JN_ANGLE;
                    currentMode = new String( "Angle Join Style" );
                    break;
                  }
                case 2: {
                    style &= ~GLE.TUBE_JN_MASK;
                    style |= GLE.TUBE_JN_CUT;
                    currentMode = new String( "Cut Join Style" );
                    break;
                  }
                case 3: {
                    mode = -1;
                    style &= ~GLE.TUBE_JN_MASK;
                    style |= GLE.TUBE_JN_ROUND;
                    currentMode = new String( "Round Join Style" );
                    break;
                  }
              }
            gle_.gleSetJoinStyle( style );
            glc.repaint();
          }

        if ( evt.getKeyChar() == 'n' || evt.getKeyChar() == 'N' ) {
            int style = gle_.gleGetJoinStyle();
            normalMode++;
            switch ( normalMode ) {
                case 0: {
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_FACET;
                    currentMode = new String( "Facet Normal Vectors" );
                    break;
                  }
                case 1: {
                    normalMode = -1;
                    style &= ~GLE.TUBE_NORM_MASK;
                    style |= GLE.TUBE_NORM_EDGE;
                    currentMode = new String( "Edge Normal Vectors" );
                    break;
                  }
              }
            gle_.gleSetJoinStyle( style );
            glc.repaint();
          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }

    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
